/*
 * Copyright 1994 Steve Passe
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of AUTHOR (Steve Passe)
 * are not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  AUTHORS make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * 1996 Revision
 * AUTHORS:     Steve Passe <smp@csn.net>
 */


static char rcsid[] = "$Id: xfocus.c,v 1.5 1995/02/03 22:15:56 smp Exp smp $";

#include <stdio.h>
#include <stdlib.h>

#define XLIB_ILLEGAL_ACCESS
/**#include <X11/Intrinsic.h>*/

#include <Xm/Xm.h>
#include <Xm/ToggleB.h>
#include <Mrm/MrmPublic.h>

#include "focus.h"

/* instance & class names for X resource access */
static char* appName = "xFocus";
static char* appClass = "XFocus";

/* Motif Resource Manager stuff */
static MrmHierarchy mrmId;
static char *mrmFile[]={"/usr/X11R6/lib/X11/uid/xfocus.uid"};
static MrmCode mrmClass;



/* callback functions */
static void pushBCB( Widget, int*, XmToggleButtonCallbackStruct* );
static void exitCB( Widget, XtPointer, XtPointer );

/* callback registration list for MRM */
static MRMRegisterArg mrmNames[] = {
        { "pushBCB",	(XtPointer)pushBCB },
        { "exitCB",	(XtPointer)exitCB }
};

/*
 * set the fallback resources BY CLASS
 */
static String fallbackResources[] = {
"XFocus*XmText.columns: 10",
"XFocus*XmTextField.columns: 10",
"XFocus*fontList: *-*-Helvetica-medium-r-*-*-*-100-*-*-p-*-*-*",
"XFocus*titleLabel.fontList: *-*-Helvetica-bold-r-*-*-*-180-*-*-p-*-*-*",
"XFocus*labelLabel.fontList: *-*-Helvetica-bold-r-*-*-*-180-*-*-p-*-*-*",
"XFocus*menuBar*fontList: *-*-Helvetica-medium-r-*-*-*-120-*-*-p-*-*-*",
"XFocus*popupMenu*fontList: *-*-Helvetica-medium-r-*-*-*-120-*-*-p-*-*-*",
"XFocus*background: grey30",
"XFocus*XmToggleButton*background: grey60",
"XFocus*menuBar*XmPushButton*background: white",
"XFocus*XmPushButton*background: grey",
"XFocus*XmFrame.XmLabel.foreground: grey",
"XFocus*selectColor: ghostWhite",
NULL
};

static XtAppContext	appContext;
static Widget		shell;


int
main( int argc, char **argv )
{
    Widget	appMain;
    Display*	display;
    Arg		args[2];
    int		status;

    /* initialize X */
    XtToolkitInitialize();
    MrmInitialize();

    /* top level X setup */
    appContext = XtCreateApplicationContext();
    XtAppSetFallbackResources( appContext, fallbackResources );
    display = XtOpenDisplay( appContext, NULL, appName, appClass,
			NULL, 0, &argc, argv );
    if ( display == NULL ) {
	fprintf( stderr, "%s:  Can't open display\n", argv[0] );
	exit( 1 );
    }

    /* open the control port to the hardware */
    status = focusOpen( FOCUS_LOG, FOCUS_PORT, FOCUS_BAUD,
		       display->display_name );
    if ( status != OK )
    {
	fprintf( stderr, "xfocus: couldn't open focus device: %d\n\n",
		status );
	exit( 1 );
    }

    /* create the top level shell */
    XtSetArg( args[0], XmNargc, argc );
    XtSetArg( args[1], XmNargv, argv );
    shell = XtAppCreateShell( appName, appClass,
			     applicationShellWidgetClass, display, args, 2 );

    /* initialize the MRM engine */
    if (MrmOpenHierarchy( 1, mrmFile, NULL, &mrmId ) != MrmSUCCESS) exit(0);
    MrmRegisterNames( mrmNames, XtNumber(mrmNames) );
    MrmFetchWidget( mrmId, "appMain", shell, &appMain, &mrmClass );
    XtManageChild( appMain );

    /* make it all happen */
    XtRealizeWidget( shell );

    /* ...if only this was done with threads... */
    XtAppMainLoop( appContext );

    exit( 0 );
}


/*
 * process the tan pushbuttons
 */
static void
pushBCB( Widget w, int* id, XmToggleButtonCallbackStruct* cb )
{
    int	status, target;

    target = *id;

    status = focusSet( target, target );
    if ( status )
    {
	/** cheap, just do again... */
	focusSet( target, target );
    }
}


/*
 * exit button in menu bar
 */
static void
exitCB( Widget w, XtPointer cd, XtPointer cb )
{
    exit( 0 );
}
