/*
 * $Id: Gate.c,v 1.5 1995/03/29 21:55:15 smp Exp smp $
 *
 * derived from:
 * $XConsortium: Clock.c,v 1.24 91/05/22 17:20:31 converse Exp $
 *
 * Copyright 1989 Massachusetts Institute of Technology
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Gate.c
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xmu/Converters.h>
#include "GateP.h"
#include <X11/Xos.h>
#include <stdio.h>
#include <math.h>
#include <X11/extensions/shape.h>

#define offset(field) XtOffsetOf(XgateRec, xgate.field)
#define goffset(field) XtOffsetOf(WidgetRec, core.field)

static XtResource resources[] = {
    {XtNwidth, XtCWidth, XtRDimension, sizeof(Dimension),
        goffset(width), XtRImmediate, (XtPointer) 120},
    {XtNheight, XtCHeight, XtRDimension, sizeof(Dimension),
	goffset(height), XtRImmediate, (XtPointer) 120},
    {XtNbackingStore, XtCBackingStore, XtRBackingStore, sizeof (int),
    	offset (backing_store), XtRString, "default"},
    {XtNborderSize, XtCBorderSize, XtRFloat, sizeof (float),
	offset (border_size), XtRString, "0.1"},
    {XtNshapeWindow, XtCShapeWindow, XtRBoolean, sizeof (Boolean),
	offset (shape_window), XtRImmediate, (XtPointer) True},
    {XtNtransparent, XtCTransparent, XtRBoolean, sizeof (Boolean),
#if defined( TRANSPARENT )
	offset (transparent), XtRImmediate, (XtPointer) True},
#else
	offset (transparent), XtRImmediate, (XtPointer) False},
#endif /* TRANSPARENT */
    {XtNtarget, XtCTarget, XtRString, sizeof (char*),
	offset (target), XtRString, DEFAULT_TARGET},
};

#undef offset
#undef goffset

static void Initialize(), Realize(), Destroy(), Redisplay(), Resize();

# define BORDER_SIZE(w)    ((w)->xgate.border_size)
# define WINDOW_WIDTH(w)    (2.0 - BORDER_SIZE(w)*2)
# define WINDOW_HEIGHT(w)   (2.0 - BORDER_SIZE(w)*2)

static void ClassInitialize();

XgateClassRec xgateClassRec = {
    { /* core fields */
    /* superclass		*/	&widgetClassRec,
    /* class_name		*/	"Xgate",
    /* size			*/	sizeof(XgateRec),
    /* class_initialize		*/	ClassInitialize,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions			*/	NULL,
    /* num_actions		*/	0,
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	Destroy,
    /* resize			*/	Resize,
    /* expose			*/	Redisplay,
    /* set_values		*/	NULL,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	NULL,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry,
    }
};

static void ClassInitialize()
{
    XtAddConverter( XtRString, XtRBackingStore, XmuCvtStringToBackingStore,
		    NULL, 0 );
}

WidgetClass xgateWidgetClass = (WidgetClass) &xgateClassRec;

/* ARGSUSED */
static void Initialize (greq, gnew)
    Widget greq, gnew;
{
    XgateWidget w = (XgateWidget)gnew;
    XtGCMask	valuemask;
    XGCValues	myXGCV;
    int shape_event_base, shape_error_base;

    valuemask = GCForeground;

#if !defined( TRANSPARENT )
    if (w->xgate.transparent)
    {
	;
    }
    else
    {
    	myXGCV.foreground = w->core.background_pixel;
    	w->xgate.eraseGC = XtGetGC(gnew, valuemask, &myXGCV);
    }
#else
#endif /* TRANSPARENT */

    if (w->xgate.shape_window && !XShapeQueryExtension (XtDisplay (w), 
							&shape_event_base, 
							&shape_error_base))
    w->xgate.shape_window = False;
    w->xgate.shape_mask = 0;
    w->xgate.shapeGC = 0;
    w->xgate.shape_width = 0;
    w->xgate.shape_height = 0;
}

static void Resize (w)
    XgateWidget	w;
{
    XGCValues	xgcv;
    Widget	parent;
    XWindowChanges	xwc;
    int		face_width, face_height;
    int		x, y;
    Pixmap	shape_mask;

    if (!XtIsRealized((Widget) w))
	return;

    /*
     * compute desired border size
     */

    SetTransform (&w->xgate.maskt,
		  0, w->core.width,
		  w->core.height, 0,
		  -1.0, 1.0,
 		  -1.0, 1.0);

    face_width = abs (Xwidth (BORDER_SIZE(w), BORDER_SIZE(w), &w->xgate.maskt));
    face_height = abs (Xheight (BORDER_SIZE(w), BORDER_SIZE(w), &w->xgate.maskt));

    /*
     *  shape the windows and borders
     */

    if (w->xgate.shape_window) {

	SetTransform (&w->xgate.t,
			face_width, w->core.width - face_width,
			w->core.height - face_height, face_height,
			-WINDOW_WIDTH(w)/2, WINDOW_WIDTH(w)/2,
			-WINDOW_HEIGHT(w)/2, WINDOW_HEIGHT(w)/2);
    
	/*
	 * allocate a pixmap to draw shapes in
	 */

	if (w->xgate.shape_mask &&
	    (w->xgate.shape_width != w->core.width ||
	     w->xgate.shape_height != w->core.height))
	{
	    XFreePixmap (XtDisplay (w), w->xgate.shape_mask);
	    w->xgate.shape_mask = None;
	}
	
	if (!w->xgate.shape_mask)
	{
	    w->xgate.shape_mask = XCreatePixmap (XtDisplay (w), XtWindow (w),
				    w->core.width, w->core.height, 1);
	}
	shape_mask = w->xgate.shape_mask;
    	if (!w->xgate.shapeGC)
            w->xgate.shapeGC = XCreateGC (XtDisplay (w), shape_mask, 0, &xgcv);

	/* erase the pixmap */
    	XSetForeground (XtDisplay (w), w->xgate.shapeGC, 0);
    	XFillRectangle (XtDisplay (w), shape_mask, w->xgate.shapeGC,
			0, 0, w->core.width, w->core.height);
    	XSetForeground (XtDisplay (w), w->xgate.shapeGC, 1);

	/*
	 * draw the bounding shape.  Doing this first
	 * eliminates extra exposure events.
	 */

	if (w->xgate.border_size > 0.0 || !w->xgate.transparent)
	{
	    TFillArc (XtDisplay (w), shape_mask,
			    w->xgate.shapeGC, &w->xgate.maskt,
			    -1.0, -1.0,
			    2.0, 2.0,
			    0, 360 * 64);
	}

	if (w->xgate.transparent)
	{
	    if (w->xgate.border_size > 0.0)
	    {
	    	XSetForeground (XtDisplay (w), w->xgate.shapeGC, 0);
	    	TFillArc (XtDisplay (w), shape_mask,
			    	w->xgate.shapeGC, &w->xgate.t,
			    	-WINDOW_WIDTH(w)/2, -WINDOW_HEIGHT(w)/2,
			    	WINDOW_WIDTH(w), WINDOW_HEIGHT(w),
			    	0, 360 * 64);
	    	XSetForeground (XtDisplay (w), w->xgate.shapeGC, 1);
	    }
	}
	/*
	 * Find the highest enclosing widget and shape it
	 */

	x = 0;
	y = 0;
	for (parent = (Widget) w; XtParent (parent); parent = XtParent (parent)) {
	    x = x + parent->core.x + parent->core.border_width;
	    y = y + parent->core.y + parent->core.border_width;
	}

	XShapeCombineMask (XtDisplay (parent), XtWindow (parent), ShapeBounding,
			    x, y, shape_mask, ShapeSet);

	/* erase the pixmap */
    	XSetForeground (XtDisplay (w), w->xgate.shapeGC, 0);
    	XFillRectangle (XtDisplay (w), shape_mask, w->xgate.shapeGC,
			0, 0, w->core.width, w->core.height);
    	XSetForeground (XtDisplay (w), w->xgate.shapeGC, 1);

	/*
	 * draw the clip shape
	 */

#if !defined( TRANSPARENT )
	if (w->xgate.transparent)
	{
	    ;
	}
	else
	{
	    TFillArc (XtDisplay (w), shape_mask,
			    w->xgate.shapeGC, &w->xgate.t,
			    -WINDOW_WIDTH(w)/2, -WINDOW_HEIGHT(w)/2,
			    WINDOW_WIDTH(w), WINDOW_HEIGHT(w),
			    0, 360 * 64);
	}
#endif /* TRANSPARENT */

	XShapeCombineMask (XtDisplay (w), XtWindow (w), ShapeClip, 
		    0, 0, shape_mask, ShapeSet);

    } else
    {
    	/*
     	 * reconfigure the widget to split the availible
     	 * space between the window and the border
     	 */

     	if (face_width > face_height)
	    xwc.border_width = face_height;
    	else
	    xwc.border_width = face_width;
    	xwc.width = w->core.width - xwc.border_width * 2;
    	xwc.height = w->core.height - xwc.border_width * 2;
    	XConfigureWindow (XtDisplay (w), XtWindow (w),
			    CWWidth|CWHeight|CWBorderWidth,
			    &xwc);
    
    	SetTransform (&w->xgate.t,
	    0, xwc.width,
	    xwc.height, 0,
	    -WINDOW_WIDTH(w)/2, WINDOW_WIDTH(w)/2,
	    -WINDOW_HEIGHT(w)/2, WINDOW_HEIGHT(w)/2);
    }
}
 
static void Realize (gw, valueMask, attrs)
     Widget gw;
     XtValueMask *valueMask;
     XSetWindowAttributes *attrs;
{
     XgateWidget	w = (XgateWidget)gw;

    if (w->xgate.backing_store != Always + WhenMapped + NotUseful) {
     	attrs->backing_store = w->xgate.backing_store;
	*valueMask |= CWBackingStore;
    }
    if (w->xgate.transparent)
    {
	attrs->background_pixel = w->xgate.minute;
	*valueMask |= CWBackPixel;
	*valueMask &= ~CWBackPixmap;
    }
    XtCreateWindow( gw, (unsigned)InputOutput, (Visual *)CopyFromParent,
		     *valueMask, attrs );
#if !defined( TRANSPARENT )
    if (!w->xgate.transparent)
#endif /* TRANSPARENT */
      Resize (w);
}

static void Destroy (gw)
     Widget gw;
{
     XgateWidget w = (XgateWidget)gw;

#if !defined( TRANSPARENT )
     if (! w->xgate.transparent) {
	 XtReleaseGC(gw, w->xgate.eraseGC);
     }
#endif /* TRANSPARENT */

     if (w->xgate.shapeGC)
	XFreeGC(XtDisplay(gw), w->xgate.shapeGC);
     if (w->xgate.shape_mask)
	XFreePixmap (XtDisplay (w), w->xgate.shape_mask);
}

/* ARGSUSED */
static void Redisplay(gw, event, region)
     Widget gw;
     XEvent *event;
     Region region;
{
}


#if defined( ENTER_SUPPORT ) || defined( LEAVE_SUPPORT )
static int state = 0;
static int ex;
static int ey;
#endif /* ENTER_SUPPORT || LEAVE_SUPPORT */

#if defined( ENTER_SUPPORT )
void enter( Widget w, XtPointer client_data, XEvent* event, Boolean* cont )
{
    Widget p = XtParent( w );
    int x, y, outx, outy;
    unsigned int width, height, bw, d;
    Window r;
    int i;
    char* target;
    char cmd[ 128 ];
    char name[ 128 ];
    extern char focusName[];

    target = ((XgateWidget)w)->xgate.target;
    /**fprintf( stderr, "display: '%s', target: '%s'\n", focusName, target );*/

    XBell(XtDisplay(w), 0);

# if defined( XCB_SUPPORT )

    if ( strncmp( focusName, CENTER, strlen( CENTER ) ) == 0 )
    {
	sprintf( cmd, XCB_CENTER, target );
    }
    else if ( strncmp( focusName, LEFT, strlen( LEFT ) ) == 0 )
    {
	sprintf( cmd, XCB_LEFT, focusName );
    }
    else
    {
	sprintf( cmd, XCB_RIGHT, focusName );
    }

    /**fprintf( stderr, " '%s'\n", cmd );*/
    system( cmd );

# endif /* XCB_SUPPORT */

    XGetGeometry( XtDisplay (p), XtWindow (p), &r,
		 &x, &y, &width, &height, &bw, &d );

    ex = (event->xcrossing).x;
    ey = (event->xcrossing).y;

# if defined( TRANSPARENT )

    if ( state == 0 )
    {
	state = 1;
	XWarpPointer( XtDisplay (p), XtWindow (p), XtWindow (p),
		     0, 0, width, height, width/2, height/2 );

	/* hardware bug: try till it takes! */
	for ( i = 0; i < 10; ++i )
	  if ( focusNSet( target, target ) == 0 ) break;
    }
    else
    {

#  if !defined( LEAVE_SUPPORT )
	state = 0;

	if ( ex < width/2 ) outx = -(width/4);
	else outx = width+(width/4);
	XWarpPointer( XtDisplay (p), XtWindow (p), XtWindow (p),
		     0, 0, width, height, outx, ey/**height/2*/ );
#  endif /* !LEAVE_SUPPORT */

    }

# else /* TRANSPARENT */

    XWarpPointer( XtDisplay (p), XtWindow (p), XtWindow (p),
		 0, 0, width, height, width/2, height/2 );

    /* hardware bug: try till it takes! */
    for ( i = 0; i < 10; ++i )
      if ( focusNSet( target, target ) == 0 ) break;

# endif /* TRANSPARENT */
}

#endif /* ENTER_SUPPORT */


#if defined( LEAVE_SUPPORT )

void leave( Widget w, XtPointer client_data, XEvent* event, Boolean* cont )
{
    Widget p;

    int x, y, outx, outy;
    unsigned int width, height, bw, d;
    Window r;
    int status;
    char* target;

    XBell(XtDisplay(w), 0);

    p = w;
    XGetGeometry( XtDisplay (p), XtWindow (p), &r,
		 &x, &y, &width, &height, &bw, &d );

    if ( state == 0 )
    {
	return;
    }
    else
    {
	state = 0;
	if ( ex < width/2 ) outx = -(width/4);
	else outx = width+(width/4);
	XWarpPointer( XtDisplay (p), XtWindow (p), XtWindow (p),
		     0, 0, width, height, outx, height/2 );
	return;
    }
}

#endif /* LEAVE_SUPPORT */
